local m = {}
ffi.cdef[[
int CreateDirectoryA(char *path, uint64_t pattrs);
]]
local null_ptr = ffi.new("uint64_t")

local dumproot = ".\\trophydump\\"

-- change this to whatever trophies you want to dump
local trophies = {
    "cup_lg_fra_hi",
    "cup_lg_bra_hi",
    "cup_lg_che_hi",
} 

local patterns = {}
for i,trophy in ipairs(trophies) do
    patterns[#patterns + 1] = string.format("common\\demo\\prop\\%s\\", trophy)
end

local function create_dirs_for(pathname)
    local path = ""
    for name in string.gmatch(pathname, "[^\\]+\\") do
        --log(string.format("name={%s}", name))
        path = path .. name
        if path ~= pathname then
            --log(string.format("path={%s}", path))
            local cstr = ffi.new("char[?]",#path+1)
            ffi.copy(cstr,path)
            ffi.C.CreateDirectoryA(cstr, null_ptr)
        end
    end
end

function m.read(ctx, filename, addr, len, total_size, offset)
    for i,pattern in ipairs(patterns) do
        if string.match(filename, pattern) then
            -- try opening the file
            local full_filename = dumproot .. filename
            local f = io.open(full_filename, "r+b")
            if not f then
                f = io.open(full_filename, "w+b")
                if not f then
                    -- cannot open the file: perhaps directories do not
                    -- exist yet. Create them
                    create_dirs_for(full_filename)
                    -- try again
                    f = assert(io.open(full_filename, "w+b"),
                        "PROBLEM: cannot open file for writing: " .. full_filename)
                end
            end
            log(string.format("offset: %s", offset))
            log(string.format("seeking 0x%x offset", offset))
            f:seek("set", offset)

            -- addr is actually a pointer to data in memory, so if we want
            -- to use this data, we need to make a copy of it:
            log("getting bytes...")
            local bytes,err = memory.read(addr, len)
            log(string.format("got bytes: %d (err=%s)", #bytes,  err))

            -- write to disk
            f:write(bytes)
            f:close()
            log(string.format("saved: %s", full_filename))
            break
        end
    end
end

function m.init(ctx)
    if dumproot:sub(1,1) == "." then
        dumproot = ctx.sider_dir .. dumproot
    end
    ctx.register("livecpk_read", m.read)
end

return m
